
#----------------------------------------------------------------------BP

set BP(frame-name) BP
set BP(frame-title) "Background/Peakfit Options"
set BP(icon-title) optBP

set BP(backtypenames) {none flatline slopelin flatsmoo slopesmo polynomi exponent}
set BP(peaktypenames) {none gauss skewgaus}
set BP(weightnames) {none count equal equalcon}
set BP(shapenames) {differen same keep}

set BP(backtypes) {"none" "flat line" "slope line" "flat smooth" "slope smooth" "polynomial" "exponential"}
set BP(peaktypes) {"none" "gaussian" "skewed gaussian"}
set BP(weights) {"none" "count" "equal" "equalcon"}
set BP(shapes) {"different" "same" "keep"}
set BP(values) {" " "1" "2" "3"}
set BP(backtype) "flat line"
set BP(peaktype) "gaussian"
set BP(weight) "count"
set BP(shape) "different"
set BP(shapesfile) "no"
set BP(peaks) 1
set BP(order) 1


proc BP.make-frame {} { 
   global env BP

   catch {BP.unpack}

   :frame $BP(frame-name) :label "$BP(frame-title)" :show-footer F :toplevel T \
        :icon-image "@$env(ICONHOME)/spectrum"  \
           :icon-name $BP(icon-title) :resize F

   :panel BP-OPTIONS :layout T :padx 5 :pady 5
    :layout H; :row-gap 5

   :message    BP-L1  :label "Background type:"  :valuex 15c
   :menubutton BP-BTYPE  :label "$BP(backtype)" :w 15 :strings $BP(backtypes) :command "BP.opt backtype"
   :next-row T
   :message    BP-L2  :label "Peak type:"        :valuex 15c  
   :menubutton BP-PTYPE  :label "$BP(peaktype)" :w 15 :strings $BP(peaktypes) :command "BP.opt peaktype"
   :next-row T
   :message    BP-L3  :label "Weighting:"        :valuex 15c
   :menubutton BP-WEIGHT :label "$BP(weight)"   :w 15 :strings $BP(weights)   :command "BP.opt weight"
   :next-row T
   :message    BP-L4  :label "Shape constraint:" :valuex 15c
   :menubutton BP-SHAPE  :label "$BP(shape)"    :w 15 :strings $BP(shapes)    :command "BP.opt shape"
   :next-row T
   :message    BP-L5  :label "Use shapes file:" :valuex 15c
   :menubutton BP-SHAPEFILE  :label "$BP(shapesfile)"    :w 15 :strings {"no" "yes"}    :command "BP.opt shapesfile"
   :next-row T
   :message    BP-L6  :label "Distinct peaks:"   :valuex 15c 
   :menubutton BP-PEAKS  :label "$BP(peaks)"    :w 15 :strings $BP(values)    :command "BP.opt peaks"
   :next-row T
   :message    BP-L7  :label "Polynomial order:" :valuex 15c 
   :menubutton BP-ORDER  :label "$BP(order)"    :w 15 :strings $BP(values)    :command "BP.opt order"

   :panel BP-BUTTONS :layout B :padx 5
    :layout H
   :button BP-SAVE  :label "Save as defaults" :command BP.save
    :hskip 75
   :button BP-HELP  :label Help :command click-help

   :show T
}

proc BP.opt {opt args} {
   global BP
   set BP($opt) [second $args]
   :modify [first $args] :label [second $args]
   BP.announce
}

proc BP.unpack {} {
   global BP
   foreach p [get-eg-option backpeak] {
      switch [first $p] {
      backtype {set BP(backtype) [lindex $BP(backtypes) [lsearch $BP(backtypenames) [second $p]]]}
      peaktype {set BP(peaktype) [lindex $BP(peaktypes) [lsearch $BP(peaktypenames) [second $p]]]}
      weight {set BP(weight) [lindex $BP(weights) [lsearch $BP(weightnames) [second $p]]]}
      shape {set BP(shape) [lindex $BP(shapes) [lsearch $BP(shapenames) [second $p]]]}
      shapesfile {set BP(shapesfile) [second $p]}
      peaks {set BP(peaks) [second $p]}
      order {set BP(order) [second $p]}
      }
   }
}

proc BP.announce {} {
   global BP env
   set-eg-option backpeak [list \
        [list backtype [lindex $BP(backtypenames) [lsearch $BP(backtypes) $BP(backtype)]]] \
        [list peaktype [lindex $BP(peaktypenames) [lsearch $BP(peaktypes) $BP(peaktype)]]] \
        [list weight [lindex $BP(weightnames) [lsearch $BP(weights) $BP(weight)]]] \
        [list shape [lindex $BP(shapenames) [lsearch $BP(shapes) $BP(shape)]]] \
        [list shapesfile $BP(shapesfile)] \
        [list peaks $BP(peaks)] \
        [list order $BP(order)]]
}

proc BP.save {} {edit-defaults backpeak "set-eg-option backpeak \{[get-eg-option backpeak]\}"}
